package sun.awt.windows;

import sun.java2d.opengl.WGLSurfaceData;
import sun.java2d.d3d.D3DSurfaceData;
import sun.java2d.pipe.RenderQueue;
import sun.java2d.InvalidPipeException;
import sun.java2d.pipe.BufferedContext;
import sun.java2d.pipe.hw.AccelSurface;
import sun.java2d.Surface;
import java.awt.image.DataBufferInt;
import sun.awt.image.BufImgSurfaceData;
import sun.java2d.DestSurfaceProvider;
import java.awt.image.VolatileImage;
import java.awt.image.BufferedImage;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.Color;
import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.GraphicsConfiguration;
import sun.java2d.pipe.hw.AccelGraphicsConfig;
import java.awt.Window;

abstract class TranslucentWindowPainter
{
    protected Window window;
    protected WWindowPeer peer;
    private static final boolean forceOpt;
    private static final boolean forceSW;
    
    public static TranslucentWindowPainter createInstance(WWindowPeer wWindowPeer) {
        GraphicsConfiguration graphicsConfiguration = wWindowPeer.getGraphicsConfiguration();
        if (!TranslucentWindowPainter.forceSW && graphicsConfiguration instanceof AccelGraphicsConfig) {
            String simpleName = graphicsConfiguration.getClass().getSimpleName();
            if ((((AccelGraphicsConfig)graphicsConfiguration).getContextCapabilities().getCaps() & 0x100) != 0x0 || TranslucentWindowPainter.forceOpt) {
                if (simpleName.startsWith("D3D")) {
                    return new VIOptD3DWindowPainter(wWindowPeer);
                }
                if (TranslucentWindowPainter.forceOpt && simpleName.startsWith("WGL")) {
                    return new VIOptWGLWindowPainter(wWindowPeer);
                }
            }
        }
        return new BIWindowPainter(wWindowPeer);
    }
    
    protected TranslucentWindowPainter(WWindowPeer peer) {
        this.peer = peer;
        this.window = (Window)peer.getTarget();
    }
    
    protected abstract Image getBackBuffer(boolean p0);
    
    protected abstract boolean update(Image p0);
    
    public abstract void flush();
    
    public void updateWindow(boolean b) {
        boolean update = false;
        Image image = this.getBackBuffer(b);
        while (update == 0) {
            if (b) {
                Graphics2D g = (Graphics2D)image.getGraphics();
                try {
                    this.window.paintAll(g);
                }
                finally {
                    g.dispose();
                }
            }
            update = this.update(image);
            if (update != 0) {
                continue;
            }
            b = true;
            image = this.getBackBuffer(true);
        }
    }
    
    private static final Image clearImage(Image image) {
        Graphics2D graphics2D = (Graphics2D)image.getGraphics();
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        graphics2D.setComposite(AlphaComposite.Src);
        graphics2D.setColor(new Color(0, 0, 0, 0));
        graphics2D.fillRect(0, 0, width, height);
        return image;
    }
    
    static Image access$000(Image image) {
        return clearImage(image);
    }
    
    static {
        forceOpt = (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.twp.forceopt", "false")));
        forceSW = (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.twp.forcesw", "false")));
    }
    
    private static class BIWindowPainter extends TranslucentWindowPainter
    {
        private BufferedImage backBuffer;
        
        protected BIWindowPainter(WWindowPeer wWindowPeer) {
            super(wWindowPeer);
        }
        
        @Override
        protected Image getBackBuffer(boolean b) {
            int width = this.window.getWidth();
            int height = this.window.getHeight();
            if (this.backBuffer == null || this.backBuffer.getWidth() != width || this.backBuffer.getHeight() != height) {
                this.flush();
                this.backBuffer = new BufferedImage(width, height, 3);
            }
            return b ? ((Image)TranslucentWindowPainter.access$000(this.backBuffer)) : this.backBuffer;
        }
        
        @Override
        protected boolean update(Image image) {
            VolatileImage volatileImage = null;
            if ((image instanceof BufferedImage)) {
                BufferedImage bufferedImage2 = (BufferedImage)image;
                this.peer.updateWindowImpl(((DataBufferInt)bufferedImage2.getRaster().getDataBuffer()).getData(), bufferedImage2.getWidth(), bufferedImage2.getHeight());
                return true;
            }
            if ((image instanceof VolatileImage)) {
                volatileImage = (VolatileImage)image;
                if ((image instanceof DestSurfaceProvider)) {
                    Surface destSurface = ((DestSurfaceProvider)image).getDestSurface();
                    if ((destSurface instanceof BufImgSurfaceData)) {
                        int width = volatileImage.getWidth();
                        int height = volatileImage.getHeight();
                        this.peer.updateWindowImpl(((DataBufferInt)((BufImgSurfaceData)destSurface).getRaster(0, 0, width, height).getDataBuffer()).getData(), width, height);
                        return true;
                    }
                }
            }
            BufferedImage bufferedImage = (BufferedImage)TranslucentWindowPainter.access$000(this.backBuffer);
            this.peer.updateWindowImpl(((DataBufferInt)bufferedImage.getRaster().getDataBuffer()).getData(), bufferedImage.getWidth(), bufferedImage.getHeight());
            return volatileImage == null || !volatileImage.contentsLost();
        }
        
        @Override
        public void flush() {
            if (this.backBuffer != null) {
                this.backBuffer.flush();
                this.backBuffer = null;
            }
        }
    }
    
    private static class VIWindowPainter extends TranslucentWindowPainter.BIWindowPainter
    {
        private VolatileImage viBB;
        
        protected VIWindowPainter(WWindowPeer wWindowPeer) {
            super(wWindowPeer);
        }
        
        @Override
        protected Image getBackBuffer(boolean b) {
            int width = this.window.getWidth();
            int height = this.window.getHeight();
            GraphicsConfiguration graphicsConfiguration = this.peer.getGraphicsConfiguration();
            if (this.viBB == null || this.viBB.getWidth() != width || this.viBB.getHeight() != height || this.viBB.validate(graphicsConfiguration) == 2) {
                this.flush();
                if ((graphicsConfiguration instanceof AccelGraphicsConfig)) {
                    this.viBB = ((AccelGraphicsConfig)graphicsConfiguration).createCompatibleVolatileImage(width, height, 3, 2);
                }
                if (this.viBB == null) {
                    this.viBB = graphicsConfiguration.createCompatibleVolatileImage(width, height, 3);
                }
                this.viBB.validate(graphicsConfiguration);
            }
            return b ? TranslucentWindowPainter.access$000(this.viBB) : this.viBB;
        }
        
        @Override
        public void flush() {
            if (this.viBB != null) {
                this.viBB.flush();
                this.viBB = null;
            }
        }
    }
    
    private abstract static class VIOptWindowPainter extends TranslucentWindowPainter.VIWindowPainter
    {
        protected VIOptWindowPainter(WWindowPeer wWindowPeer) {
            super(wWindowPeer);
        }
        
        protected abstract boolean updateWindowAccel(long p0, int p1, int p2);
        
        @Override
        protected boolean update(Image image) {
            if ((image instanceof DestSurfaceProvider)) {
                Surface destSurface = ((DestSurfaceProvider)image).getDestSurface();
                if ((destSurface instanceof AccelSurface)) {
                    int width = image.getWidth(null);
                    int height = image.getHeight(null);
                    boolean[] array = { false };
                    AccelSurface surface = (AccelSurface)destSurface;
                    RenderQueue renderQueue = surface.getContext().getRenderQueue();
                    renderQueue.lock();
                    try {
                        BufferedContext.validateContext(surface);
                        renderQueue.flushAndInvokeNow(new Runnable() {
                            @Override
                            public void run() {
                                array[0] = TranslucentWindowPainter.VIOptWindowPainter.this.updateWindowAccel(surface.getNativeOps(), width, height);
                            }
                        });
                    }
                    catch (InvalidPipeException ex) {}
                    finally {
                        renderQueue.unlock();
                    }
                    return array[0];
                }
            }
            return super.update(image);
        }
    }
    
    private static class VIOptD3DWindowPainter extends TranslucentWindowPainter.VIOptWindowPainter
    {
        protected VIOptD3DWindowPainter(WWindowPeer wWindowPeer) {
            super(wWindowPeer);
        }
        
        @Override
        protected boolean updateWindowAccel(long n, int n2, int n3) {
            return D3DSurfaceData.updateWindowAccelImpl(n, this.peer.getData(), n2, n3);
        }
    }
    
    private static class VIOptWGLWindowPainter extends TranslucentWindowPainter.VIOptWindowPainter
    {
        protected VIOptWGLWindowPainter(WWindowPeer wWindowPeer) {
            super(wWindowPeer);
        }
        
        @Override
        protected boolean updateWindowAccel(long n, int n2, int n3) {
            return WGLSurfaceData.updateWindowAccelImpl(n, this.peer, n2, n3);
        }
    }
}
